package ch.iet_gibb.heatcalculatorfx.model;

import java.util.ArrayList;

/**
 * Superklasse für Wärmespeicher
 *
 * @author Benedikt Schwiede
 * @version 1.0.0
 * @since 04.03.2025
 */
public abstract class GeneralHeatstorage implements HeatstorageInterface {
    /**
     * Heizkapazität von Wasser
     */
    protected double SPECIFIC_HEAT_WATER_KWH = 4.18 / 3600;
    /**
     * Bezeichnung des Wärmespeichers
     */
    protected String name;
    /**
     * Volumen des Wärmespeichers
     */
    protected float volume;
    /**
     * Temperatur des Wärmespeichers
     */
    protected float temperature;
    /**
     * Energie die täglich benötigt wird
     */
    protected float requiredEnergyPerDay;

    /**
     * Eigenschaften des Wärmespeichers
     */
    protected ArrayList<Property> properties;

    protected GeneralHeatstorage() {
    }

    /**
     * Konstruktor für Superklasse
     *
     * @param name
     * @param volume
     * @param temperature
     * @param requiredEnergyPerDay
     */
    protected GeneralHeatstorage(String name, float volume, float temperature, float requiredEnergyPerDay) {
        this.name = name;
        this.volume = checkValue(0, volume);
        this.temperature = checkValue(30, temperature);
        this.requiredEnergyPerDay = checkValue(0, requiredEnergyPerDay);
        this.properties = new ArrayList<>();

        this.properties.add(new Property("Name", name, "String"));
        this.properties.add(new Property("Volume", String.valueOf(volume), "float"));
        this.properties.add(new Property("Temperature", String.valueOf(temperature), "float"));
        this.properties.add(new Property("Required Energy Per Day", String.valueOf(requiredEnergyPerDay), "float"));
        this.properties.add(new Property("Stored Energy", String.valueOf(getStoredEnergy()), "float"));
        this.properties.add(new Property("Remaining Days", String.valueOf(getRemainingDaysOfEnergy()), "float"));
    }

    /**
     * @return Bezeichnung des Wärmespeichers
     */
    public String getName() {
        return name;
    }

    /**
     * @param name Bezeichnung des Wärmespeichers
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return Volumen des Wärmespeichers
     */
    public float getVolume() {
        return volume;
    }

    /**
     * @param volume Volumen des Wärmespeichers
     */
    public void setVolume(float volume) {
        this.volume = checkValue(0, volume);
    }

    /**
     * @return Temperatur des Wärmespeichers
     */
    public float getTemperature() {
        return temperature;
    }

    /**
     * @param temperature Temperatur des Wärmespeichers
     */
    public void setTemperature(float temperature) {
        this.temperature = checkValue(30, temperature);
    }

    /**
     * @return Benötigte Energie pro Tag
     */
    public float getRequiredEnergyPerDay() {
        return requiredEnergyPerDay;
    }

    /**
     * @param requiredEnergyPerDay Benötigte Energie pro Tag
     */
    public void setRequiredEnergyPerDay(float requiredEnergyPerDay) {
        this.requiredEnergyPerDay = checkValue(0, requiredEnergyPerDay);
    }

    /**
     * @return Gespeicherte Energie
     */
    public float getStoredEnergy() {
        return (int) (getVolume() * SPECIFIC_HEAT_WATER_KWH * (temperature - 23));
    }

    /**
     * @return Tage bis Wärmespeicher leer ist
     */
    public int getRemainingDaysOfEnergy() {
        return (int) (getStoredEnergy() / requiredEnergyPerDay);
    }

    /**
     * Berechnet Volumen des Wärmespeichers mittels Durchmesser und Höhe
     *
     * @param diameter
     * @param height
     * @return Volumen des Wärmespeichers
     */
    protected static float calculateVolumeWithDiameterAndHeight(float diameter, float height) {
        return (float) (Math.PI * Math.pow((diameter / 2), 2) * height) / 1000;
    }

    protected float checkValue(int minValue, float valueToCheck) {
        if (valueToCheck < minValue) {
            throw new IllegalArgumentException("Value must be greater than " + minValue);
        }
        return valueToCheck;
    }

    public ArrayList<Property> getProperties() {
        return properties;
    }
}
